/*
 * Phone dialing Module
 *
 *
 *	ST users @ Case Western Reserve University
 */

#include <osbind.h>
#include <ctype.h>
#include "xmdm.h"

#define PREDIAL		"ATDT"	/* Modems dial command */
#define clear_screen()  Cconws("\033H\033J")
#define inv()		EscSeq(p)
#define normal()	EscSeq(q)
#define mvto(r,c)	EscSeq(Y);Cconout(r+040);Cconout(c+040)
#define ceol()		EscSeq(K)
#define show_cursor()	EscSeq(e)
#define hide_cursor()	EscSeq(f)

typedef struct _dir {		/* The Directory type */
	char	*name;		/* Name (11 chars max) */
	char 	*number;	/* Phone # (24 chars max) */
	int	baud;		/* baud Rate		 */
	struct _dir *next;	/* Ptr to next entry */
} *DIR;


 	/* External Variables */
extern char scratch[];		/* Buffer */
extern int currb;		/* Current Baud Rate */
extern int dchanged;		/* Has the directory been updated */
extern char capture;		/* capture flag */
 	/* Globals */
static int ndir = 0;			/* # of entries in phone directory   */
static char *pre_dial = PREDIAL; 	/* The Pre Dial code		     */
static DIR directory = (DIR)NULL;	/* The phone directory  	     */
static DIR lastdir   = (DIR)NULL; 	/* Pointer to last entry	     */
static char *dirfile = (char *)NULL;	/* Name of file conatining directory */
static int rflag = 0;			/* Read directory as yet?? 	     */


/*
 * Read the Telephone directory
 *  returns -2 on read error
 *	    -1 if cancelled
 *	    -3 if file not found
 *	     n # of entries  otherwise
 */
int readdir()
{
	register char *filename;
	register int handle;
	register long nbytes;
	register int nentries;
	register DIR last;
	register DIR present;
	extern char *readline();
	extern char *malloc();
	
	Cconws("Enter Filename of Phone Directory or <CR> to Cancel: ");

	if((filename = readline()) == (char *)NULL)
	    /* Cancelled */
	    return -1;

	if((dirfile = malloc(strlen(filename)+1)) == (char *)NULL)
	{
		/* Out of memory */
		Cconws("Out of Memory\r\n");
		return 0;
	}
	strcpy(dirfile,filename);
	

	if((handle = Fopen(filename,0)) < 0)
	    /* File does not exist */
	    return -3;
	
	/* Read in the file */
	if((nbytes = Fread(handle, 2L, &ndir)) != 2L)
	{
		Cconws("Error Reading ");
		Cconws(filename);
		Cconws("\r\n");
		Fclose(handle);
		return -2;
	}

	/* Read in the directory */
	last = (DIR)NULL;
	directory = (DIR)NULL;
	lastdir = (DIR)NULL;
	rflag = 1;
	
	for(nentries = 0; nentries < ndir; nentries++)
	{
		/* Allocate an entry */
		if((present = (DIR)malloc(sizeof(struct _dir))) == (DIR)NULL)
		{
			/* Out of memory */
			Cconws("Out of Memory - Read ");
			printi(nentries);
			Cconws(" Entry(s)\r\n");
			Fclose(handle);
			return nentries;
		}
		
		if((present->name = malloc(12)) == (char *)NULL)
		{
			/* Out of memory */
			Cconws("Out of Memory - Read ");
			printi(nentries);
			Cconws(" Entry(s)\r\n");
			Fclose(handle);
			return nentries;
		}

		if((present->number = malloc(25)) == (char *)NULL)
		{
			/* Out of memory */
			Cconws("Out of Memory - Read ");
			printi(nentries);
			Cconws(" Entry(s)\r\n");
			Fclose(handle);
			return nentries;
		}

		present->next = (DIR)NULL;
		
		/* Read in the entry */
		if((nbytes = Fread(handle,11L,present->name)) != 11L)
		{
			Cconws("Error Reading ");
			Cconws(filename);
			Cconws("\r\n");
			Fclose(handle);
			rflag = 0;
			freedir(directory);
			return -2;

		}

		if((nbytes = Fread(handle,24L,present->number)) != 24L)
		{
			Cconws("Error Reading ");
			Cconws(filename);
			Cconws("\r\n");
			Fclose(handle);
			rflag = 0;
			freedir(directory);
			return -2;
		}

		if((nbytes = Fread(handle,2L,&(present->baud))) != 2L)
		{
			Cconws("Error Reading ");
			Cconws(filename);
			Cconws("\r\n");
			Fclose(handle);
			rflag = 0;
			freedir(directory);
			return -2;
		}
		

		present->name[11] = '\0';
		present->number[24] = '\0';

		/* Link it on with the directory */
		if(last == (DIR)NULL)
			/* first entry */
			directory = present;
		else
			last->next = present;
		
		last = present;
	}
	lastdir = last;
	
	return nentries;	
}

/*
 * Free space allocated to a phone directory
 *
 */
void freedir(dir)
register DIR dir;
{
	register DIR next;
	register DIR present;

	for(present = dir; present != (DIR)NULL; present = next)
	{
		next = present->next;
		free(present->name);
		free(present->number);
		free(present);
	}
	
	if(dirfile != (char *)NULL)
	{
		free(dirfile);
		dirfile   = (char *)NULL;
	}
	
	directory = (DIR)NULL;
	lastdir	  = (DIR)NULL;
	ndir	  = 0;
	rflag	  = 0;
}

/*
 * Write out the phone directory
 *  -returns 0 on success 1 otherwise
 *
 */
int writedir()
{
	register DIR dir;
	register int fd;
	
	if((rflag == 0) || (dirfile == (char *)NULL) || (dchanged == 0))
	    /* Nothing to Save */
	    return 0;

	/* Create/Open file for write - overwrite if it exists */
	if ((fd = Fcreate(dirfile,0)) < 0) /* Will fail is file present */
	{
		/* Overwrite existing file */
		if((fd = Fopen(dirfile,1)) < 0)
		{
			/* CAN the transfer if we cannot open file */
			Cconws("Cannot Open ");
			Cconws(dirfile);
			Cconws("\r\n");
			return 1;
		}
	}

	if(Fwrite(fd,2L,&ndir) != 2L)
	{
		Cconws("Error Writing ");
		Cconws(dirfile);
		Cconws("\r\n");
		Fclose(fd);
		return 1;
	}
	
	for(dir = directory; dir != (DIR)NULL; dir = dir->next)
	{
		if(Fwrite(fd,11L,dir->name) != 11L)
		{
			Cconws("Error Writing ");
			Cconws(dirfile);
			Cconws("\r\n");
			Fclose(fd);
			return 1;
		}

		if(Fwrite(fd,24L,dir->number) != 24L)
		{
			Cconws("Error Writing ");
			Cconws(dirfile);
			Cconws("\r\n");
			Fclose(fd);
			return 1;
		}

		if(Fwrite(fd,2L,&(dir->baud)) != 2L)
		{
			Cconws("Error Writing ");
			Cconws(dirfile);
			Cconws("\r\n");
			Fclose(fd);
			return 1;
		}
	}
	
	Fclose(fd);
	return 0;
}

/*
 * Show the phone directory
 * return the entry # or -1 if cancelled
 *
 */
int showdir()
{
	register int first, last;
	register int n;
	register char *line;
	extern int atoi();
	extern char *readline();
	
	first = 0;

	while(1)
	{
	    again:
		clear_screen();

		mvto(0,19);
		Cconws("Phone Directory: ");
		Cconws(dirfile);
		Cconws("  ");
		printi(ndir);
		Cconws(" Entry(s)");

		last = (ndir < (first + 44)) ? ndir : first + 44;
		putdir(first,last);
		
		mvto(25,0);
		inv();
		Cconws("Enter a # or <SPACE><RETURN> for Next Page or <RETURN> to Cancel:");
		normal();
		Cconout(' ');
		if((line = readline()) == (char *)NULL)
		{
			return -1;
		}
		
		if(isdigit(*line))
		{
			if((n = atoi(line)) > ndir)
			{
				Cconws("Invalid Number ");
				hit_key();
				goto again;
			}
		    
			return n;
		}

		if(last == ndir)
		    first = 0;
		else
		    first += 44;
	}
}

/*
 * Put up directory entries on the screen
 *
 */
void putdir(first,last)
register int first;
register int last;
{
	register DIR dir;
	register int row;
	extern DIR nth();
	
	/* Find the first entry */
	dir = nth(first);
	row = (first % 44) + 1;

	hide_cursor();
	inv();
	for(; first < last; first++)
	{
		mvto(row,((first & 1)?41:0));
		if(first < 10)
			Cconout(' ');
		printi(first);
		Cconout('|');
		putstr(dir->name,11);
		Cconout('|');
		putstr(dir->number,24);
		dir = dir->next;
		if(first & 1)
			row++;
	}

	if(first & 1)
	{
		mvto(row,41);
		Cconws("  |           |                        ");
		row++;
	}

	for(; row < 23; row++)
	{
		Cconws("  |           |                        ");
		mvto(row,41);
		Cconws("  |           |                        ");
	}

	normal();
	show_cursor();
}

/*
 * Put a string padding to len on screen
 */
void putstr(s,l)
register char *s;
register int l;
{
	register int pad;

	Cconws(s);
	if((pad = l - strlen(s)) <= 0)
		return;
	for(; pad > 0; pad--)
		Cconout(' ');
}


/*
 * Return the nth entry in the phone directory
 *
 */
DIR nth(n)
register int n;
{
	register int i;
	register DIR dir;
	
	for(i = 0, dir = directory; (dir != (DIR)NULL) & (i < n);
	    i++, dir = dir->next)
	    /* Skip */;
	return(dir);
}

/*
 * Add a entry in the phonebook
 */
addir()
{
	register DIR dir;
	register char *s;
	extern char *malloc(), *readline();
	
	/* If a directory file already exists add, else read or create */
	if(rflag == 0)
	{
		switch(readdir())
		{
		    case -3:
			/* Doesnt exist, but we will create it when we
			   write out the directory */
			rflag = 1;
			break;
			
		    case 0:
		    case -2:
		    case -1:
			hit_key();
			return;
			
		    default:
			rflag = 1;
			break;
		}
	}
	
	/* Allocate space for the new entry */
	if((dir = (DIR)malloc(sizeof(struct _dir))) == (DIR)NULL)
	{
		Cconws("Out of Memory\r\n");
		hit_key();
		return;
	}
	
	if((dir->name = malloc(12)) == (char *)NULL)
	{
		/* Out of memory */
		Cconws("Out of Memory\r\n");
		hit_key();
		return;
	}

	if((dir->number = malloc(25)) == (char *)NULL)
	{
		/* Out of memory */
		Cconws("Out of Memory\r\n");
		hit_key();
		return;
	}

		
	/* Get the entry */
	
	do {
		Cconws("Name: ");
		s = readline();
	} while(s == (char *)NULL);
	
	strncpy(dir->name,s,11);
	
	do {
		Cconws("Number: ");
		s = readline();
	} while(s == (char *)NULL);
	
	strncpy(dir->number,s,24);
	
	do {
		Cconws("Baud Rate: ");
		s = readline();
	} while((s == (char *)NULL) || ((dir->baud = tobaud(s)) == -1));
	
	dir->next = (DIR)NULL;
	
	if(directory == (DIR)NULL)
	    directory = dir;
	else
	    lastdir->next = dir;

	lastdir = dir;
	dchanged = 1;
	ndir++;
}

static struct {
	char *sbaud;
	int  ibaud;
} vbauds[] = {
    { "19200", 0 },
    { "9600" , 1 },
    { "4800" , 2 },
    { "2400" , 4 },
    { "1200" , 7 },
    { "300"  , 9 },
    { (char *)NULL, -1 }
};

/*
 * Convert a string to a baud rate
 * return int or -1 if invalid
 */
int tobaud(s)
register char *s;
{
	register int i;
	
	for(i = 0; vbauds[i].sbaud != (char *)NULL; i++)
	{
		if(strcmp(vbauds[i].sbaud,s) == 0)
		    return vbauds[i].ibaud;
	}

	Cconws(s);
	Cconws(": Invalid Baud Rate\r\nValid Baud Rates are:\r\n");
	for(i = 0; vbauds[i].sbaud != (char *)NULL; i++)
	{
		Cconout('\t');
		Cconws(vbauds[i].sbaud);
		Cconws("\r\n");
	}
		
	return -1;
}

/*
 * Dial a number 
 */
dial()
{
	register int n;
	register DIR dir;
	extern DIR nth();
	
	/* Has the directory been read so far */
	if(rflag == 0)
	{
		/* Go read it */
		switch(readdir())
		{
		    case -1:
		    case -2:
		    case -3:
		    case 0:
			rflag = 0;
			hit_key();
			his_screen();
			return;
		    default:
		}
	}
	
	if((n = showdir()) == -1)
	{
		/* Cancelled */
		hit_key();
		his_screen();
		return;
	}
	
	his_screen();
	dir = nth(n);
	if(dir->baud != currb)
	{
		currb = dir->baud;
		Rsconf(currb, -1, -1, -1, -1, -1);
		sendchar('\r');
		flushinput();
	}
	write_modem(pre_dial,strlen(pre_dial));
	write_modem(dir->number,strlen(dir->number));
	sendchar('\r');
}

/*
 * Open a phone directory 
 *
 */
opendir()
{
	/* if one is open, save it if changed, then deallocate memory
 	 * and then open a new directory
	 */
	if(rflag)
	{
		if(dchanged)
		{
			if(writedir() == 1)
			{
				hit_key();
				return;
			}
			dchanged = 0;
		}
		freedir(directory);
	}
	
	if(readdir() <= 0)
	{
		hit_key();
		return;
	}
}

/*
 * Delete an entry.
 */
delentry()
{
	register DIR dir, del;
	register int n;
	extern DIR nth();
	
	if(rflag == 0)
	{
		Cconws("Nothing to delete\r\n");
		hit_key();
		return;
	}
	
	if((n = showdir()) == -1)
	{
		/* Cancelled */
		hit_key();
		return;
	}
	
	if(ndir == 1)
	{
		del = directory;
		directory = (DIR)NULL;
		lastdir = (DIR)NULL;
	}
	else
	{
		if(n == 0)
		{
			del = directory;
			dir = directory->next;
			directory = dir;
		}
		else
		{
			dir = nth(n-1);
			del = dir->next;
			dir->next = del->next;
			if(lastdir == del)
			    lastdir = dir;
		}
	}
	
	free(del->name);
	free(del->number);
	free(del);
	ndir--;
	dchanged = 1;
	
}

	
/*
 * Phone services - top level
 *
 */
phone()
{
	register long conin;
	
	while(1)
	{
		clear_screen();
		mvto(2,32);
		inv();
		Cconws("Phone  Services");
		normal();

		mvto(5,0);
		
		if(rflag)
		{
			Cconws("\tThe Phone Directory \"");
			Cconws(dirfile);
			Cconws("\" containing ");
			printi(ndir);
			Cconws(" Entry(s) is Currently Open.\r\n\n");
		}
		else
		    Cconws("\tNo Phone Directory is Currently Open.\r\n\n");
		
		
		/* Put up menu */
		Cconws("\r\n\t");
		EscSeq(p);		/* reverse video */
		Cconws("Undo");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to exit the emulator.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("d");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to dial a number.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("a");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to add an entry to the phone directory.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("D");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to delete an entry from the phone directory.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("o");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to open another phone directory.\r\n");
		
		Cconws("\t");
		EscSeq(p);		/* reverse video */
		Cconws("Return");
		EscSeq(q);		/* quit reverse video */
		Cconws(" to return to the emulator.\r\n\n\n\n");
		
		/* get response */
		conin = Bconin(2);
		
		if (conin == 0x610000)
		{
			/* He hit <UNDO> */
			if(capture)
			{
				Cconws("Closing Capture File\r\n");
				c_capture();
			}
			his_screen();
			ResetIoBuf();
			finish();
		}
		
		switch((int)(conin & 0x7f))
		{
		    case 'd':
			/* Dial a number */
			dial();
			return;
			
		    case 'D':
			/* Delete an entry */
			delentry();
			break;
			
		    case 'a':
			/* Add an entry */
			addir();
			break;
			
		    case 'o':
			/* Open another phone directory */
			opendir();
			break;
			
		    case '\r':
			his_screen();
			return;
			
		    default:
		}
		
	}
	
}
