/*
 * Utilities Module
 *
 *
 *	ST users @ Case Western Reserve University
 */

#include <setjmp.h>
#include <osbind.h>
#include "xmdm.h"

#define Vsync()	xbios(37)	/* Atari forgot these in osbind.h */
#define Supexec(X) xbios(38,X)

#define goto(r,c)	EscSeq(Y);Cconout(r+040);Cconout(c+040)

 	/* External Variables */
extern char scratch[];	/* Pointer to scratch buffer	*/
extern long *ms_ptr;	/* pointer to my screen memory aligned to a 256
			   byte boundary */

extern jmp_buf abort_env,	  /* Long jump when transfers are aborted */
        time_env;		  /* Long jump on timeout		  */

 	/* Globals */
static char *hs_ptr;   /* pointer to his screen memory */
static long
     alrm_time = 0L,	  /* Time of next timeout (200 Hz) */
     pr_time   = 0L,	  /* Present time	  (200 Hz) */
     *hz_200 = (long *)0x000004ba; /* Yes the Hitch Hikers Guide is wrong!! */

/*
 * my_screen() - Start using my screen memory
 * for all output.
 *
 */
void my_screen()
{
	/* The cursor position has been saved prior to calling this routine */
	/* save his screen memory pointer */
	hs_ptr = Logbase();
	
	/* switch to my display memory */
	Setscreen(ms_ptr, ms_ptr, -1);
}


void his_screen()
{
	/* switch to his Screen memory, wait for a Vblank, so
	 * that the Logical Loc == Physical Loc, then pop saved
	 * cursor */
	Setscreen(hs_ptr, hs_ptr, -1);
	Vsync();
	Vsync();	/* starts happening at the last one */
	goto(25,0);
	
}

/*
 * r_filename() - Read a file name from the console
 * giving the user a chance to cancel by hitting <RETURN>
 *
 */
char *r_filename()
{
	Cconws("Filename (<RETURN> to cancel): ");
	scratch[0] = 127;
	Cconrs(scratch);
	Cconws("\r\n");
	if(scratch[1] == 0)
	{
		/* User Cancelled */
		Cconws("Cancelled\r\n");
		return((char *)NULL);
	}
	/* Terminate string that starts at scratch[2] */
	scratch[scratch[1]+2] = '\0';
	return(&scratch[2]);
}


/*
 * hit_key() - wait for the user to hit a key
 * 
 */
void hit_key()
{
	Cconws("\r\n");
	EscSeq(p);
	Cconws("Hit Any Key To Continue .....");
	EscSeq(q);
	Cnecin();
	Cconws("\r\n");
}


/*
 * rd_time() - read the value of the systems 200 Hz counter
 * must be called in Super Mode else you get
 * what you deserve - MUSHROOMS!!
 */
void rd_time()
{
	pr_time = *hz_200;
}


/*
 * read_modem() - read a character from the modem port
 * Check for user abort (^C) and timeout at the same time
 */
int read_modem()
{
	while(1)
	{
		if(Cconis())
		{
			/* Character Hit at the Keyboard - is it ^C */
			if((Crawcin() & 0x7f) == CTRL('C'))
			    /* It is a Control-C */
			    longjmp(abort_env,-1);
		}
		if(Cauxis())
		    /* Character available at Modem Port */
		    return(Cauxin());
		
		/* Check for time out if required */
		if(alrm_time != 0)
		{
			/* Alarm Set */
			Supexec(rd_time);
			if(pr_time >= alrm_time)
			    /* timeout */
			    longjmp(time_env, -1);
		}
	}
}


/*
 * alarm() - set the alarm time
 */
void alarm(n)
unsigned int n;
{
	if(n != 0)
	{
		Supexec(rd_time);
		/* We really need n * 200 but n * 256 if close enough */
		alrm_time = pr_time + ( n << 8 );
	}
	else
	    alrm_time = 0L;
}

/*
 * write_modem() - send buffer to the modem port
 *
*/
void write_modem(buf,len)
register char *buf;
register int len;
{
	register int i;
	for(i = 0; i < len; i++)
	    Cauxout(*buf++);
}


/*
 * flushinput() - flush any characters in the modem port
 * a future enhancement may be flush any characters that are in there
 * and any that come in over the next 1 second -- Ymodem does that
 *
 */
void flushinput()
{
	while(Cauxis())
	    Cauxin();
}

/*
 * Print an integer (16 bits) to the console
 *
 */
void printi(val)
register int val;
{
        register int j;
        register int div_idx;
        register int first;
	
        static int divisors[] = { 10000, 1000, 100, 10, 1 };
	
	
	/* Special Cases */
        if (val == 0)
	{
		Cconout('0');
	}
        else if (val == -32768)
	{
                Cconws("-32768");
		
	}
	
	/* Get digit for each power of 10 and print them, skip leading 0's */
        first = div_idx  = 0;
	
        if (val < 0)
	{
                Cconout('-');
                val = -val;
	}
	
        while( div_idx < 5 )
	{
                if(((j = val / divisors[div_idx]) != 0) || first != 0)
		{
                        Cconout(j + '0');
                        first = 1;
		}
                val %= divisors[div_idx++];
	}
	
}

/*
 * Prinl - print a long integer (32 bits)
 *
 */
prinl(val)
register long val;
{
        register int j;
        register int div_idx;
        register int first;
        static long divisors[] = { 1000000000,100000000,10000000,1000000,
				   100000,10000, 1000, 100, 10, 1 };
#define MAX_IDX 10
	
        if (val == 0)
	{
		Cconout('0');
                return;
	}
        else if (val == -2147483648)
	{
                Cconws( "-2147483648");
		return;
	}
	
        if (val < 0L)
	{
		Cconout('-');
                val = -val;
	}
	
	first = 0;
        for(div_idx = 0; div_idx < MAX_IDX; div_idx++)
	{
                if(((j = (int)(val / divisors[div_idx])) != 0) || first != 0)
		{
                        Cconout(j + '0');
                        first = 1;
		}
                val %= divisors[div_idx];
	}
	
}


/*
 * find the size of a file given its name
 */
long filesize(name)
register char *name;
{
	DTA_T *dta;
	extern long c2long();
	
	/* find the size of the file */
	if((dta = Fgetdta()) == (DTA_T *)NULL)
	{
		return 0L;
		
	}

	if(Fsfirst(name,(int)(0x01 | 0x020)) != 0)
	{
		return 0L;
		
	}

	return c2long(dta->size);
}



/*
 * Char to long
 * Convert 4 bytes to a long
 *
 */
long c2long(size)
char size[];
{
	register long result, temp;
	register int i;

	result = size[0];
	for(i = 1; i < 4; i++)
	{
		temp = ((long)size[i]) & 0x000000ff;
		result = result << 8;
		result |= temp;
	}

	return(result);
}

/*
 * Read a line from Standard Input and return a 
 * NULL terminated pointer to it
 */
char *readline()
{
	/* Use the scratch bufr for storage */
	scratch[0] = 80;
	Cconrs(scratch);
	Cconws("\r\n");
	if(scratch[1] == 0)
	{
		/* User Cancelled */
		Cconws("Cancelled\r\n");
		return((char *)NULL);
	}
	/* Terminate string that starts at scratch[2] */
	scratch[scratch[1]+2] = '\0';
	return(&scratch[2]);
}	

